==> libremail-2.2.4.tar.bz2: libremail/src/simulsmtp.c <==
/*
    Fichier simulsmtp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant
    la simulation d'envoi de mails au serveur smtp
    Fonctionne mme sans connexion au rseau
*/

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "smtp.h"
#include "base64.h"


#define defaut_port_smtp  25  // port utilis par dfaut par les serveurs smtp

#define octet      unsigned char


/* Connexion au serveur smtp (mme fonction qu'en mode normal) */

int connect_smtp (char *infoserveur)
{
    octet *carinfo;       // caractre de infoserveur
    char  serv_smtp [80]; // nom du serveur smtp utilis
    int   port;           // numro du port smtp utilis
    FILE  *fich_auth;     // descripteur du fichier d'authentification smtp
    char  bufin [80];     // buffer pour lire le fichier d'authentification
    char  bufw [80];      // buffer d'envoi d'une ligne ou requte smtp
    int   caract;         // caractre du fichier d'authentification smtp
    int   retour;         // code de retour de la fonction


    // initialisation du numro de port par dfaut
    port = defaut_port_smtp;

    // si la ligne smtp de fichier de configuration fait
    // rfrence  un fichier d'authentification smtp
    if (*infoserveur == '>')
    {
        // se positionner sur le nom de ce fichier
        carinfo = infoserveur + 1;

        while (*carinfo == ' ' || *carinfo == '\t')
            carinfo ++;

        fich_auth = fopen (carinfo, "r");

        if (fich_auth)
        {
            // recuprer le nom du serveur smtp
            fgets (serv_smtp, 80, fich_auth);
            serv_smtp [strlen (serv_smtp) - 1] = '\0';

            // mmoriser les autres lignes particulires du fichier
            // d'authentification smtp (on s'arrte sur une ligne vide)
            while (fgets (bufin, 80, fich_auth) && *bufin != '\n')
            {
                bufin [strlen (bufin) - 1] = '\0';

                // numro de port spcifi ?
                if (startligne (bufin, "port"))
                {
                    // rcuprer le numro du port
                    carinfo = bufin + 4;

                    while (*carinfo == ' ' || *carinfo == '\t')
                        carinfo++;

                    port = atoi (carinfo);

                    // le vrifier sommairement
                    if (!port)
                    {
                        // "Numro de port smtp invalide"
                        affiche_err ("PORT_SMTP_INVALIDE");
                        retour = 0;
                    }
                }
            }

            // tentative de connexion au serveur smtp
            if (port)
                retour = connect_serv_smtp (serv_smtp, port);

            // si la connexion s'est bien droule
            if (retour)
            {
                // se positionner sur la premire ligne
                // du dialogue d'authentification
                while (fgets (bufin, 80, fich_auth) && *bufin == '\n')
                    ;

                // dialogue d'authentification
                do
                {
                    // mise en forme de la ligne lue
                    bufin [strlen (bufin) - 1] = '\0';

                    // si ligne  encoder base 64
                    if (startligne (bufin, "b64"))
                    {
                        // l'encoder avant de l'envoyer
                        encode64 (bufin + 4, bufw, strlen (bufin + 4));
                        puts (bufin);
                        envoie_smtp (bufw);
                    }
                    // sinon, envoyer directement la ligne
                    else
                        envoie_smtp (bufin);

                    // et lire la rponse du serveur
                    lire_smtp ();
                }
                // passage  la ligne suivante
                while (fgets (bufin, 80, fich_auth));
            }

            // librer le fichier d'authentification smtp
            fclose (fich_auth);
        }
        else
        {
            // message d'erreur
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", carinfo);
            retour = 0;
        }
    }
    // sinon simple connexion  un serveur smtp sans authentification
    else
    {
        // parcourir le nom du serveur smtp
        carinfo = infoserveur;

        while (*carinfo > ' ')
            carinfo ++;

        // si le nom du serveur smtp est suivi d'autres informations
        if (*carinfo)
        {
            // terminer le nom du serveur smtp
            *carinfo = '\0';

            // se positionner sur le numro de port ventuel
            do
                carinfo++;
            while (*carinfo == ' ' || *carinfo == '\t');
        }

        // si un numro de port semble prsent
        if (*carinfo)
            // le rcuprer
            port = atoi (carinfo);

        // si numro de port trouv ou si on a conserv le port par dfaut
        if (port)
            // tablir la connexion smtp avec le port choisi
            retour = connect_serv_smtp (infoserveur, port);

        // sinon message d'erreur
        else
        {
            // "Numro de port smtp invalide"
            affiche_err ("PORT_SMTP_INVALIDE");
            retour = 0;
        }

        // si la connexion smtp s'est bien droule
        if (retour)
        {
            // gnrer le message  helo nom_de_calculateur
            strcpy (bufw, "HELO ");
            gethostname (bufw + 5, sizeof (bufw) - 5);

            // identification de l'ordinateur connect
            envoie_smtp (bufw);
            lire_smtp ();
        }
    }

    // renvoyer le code de retour de la fonction
    return (retour);
}



/* Simulation de connexion russie au serveur smtp sans authentification */

int connect_serv_smtp (char *serveur, int port_smtp)
{
    char bufw [80];  // buffer d'envoi d'une ligne ou requte smtp


    printf ("Connexion au serveur %s sur le port %d\n", serveur, port_smtp);

    // simulation de la rponse du serveur smtp
    lire_smtp ();
    return (1);
}



/* cherche si la ligne passe en paramtre commence par un mot cl particulier */

int startligne (char *ligne, char *motcle)
{
    int i;

    i = 0;

    // on teste caractre par caractre en ignorant la casse
    while (motcle [i] && tolower (ligne [i]) == tolower (motcle [i]))
        i++;

    // le mot cl doit tre suivi d'un blanc dans la ligne lue
    return (ligne [i] == ' ');
}



/* Simulation criture de donnes */

void envoie_smtp (char *buffer)
{
    puts (buffer);
}



/* Simulation lecture d'une ligne de donnes */

void lire_smtp ()
{
    strcpy (buf_lect, "250 OK");
}



/* Simulation dconnexion propre du serveur smtp */

void deconnect_smtp ()
{
    // envoi du message de deconnexion
    envoie_smtp ("QUIT");

    // lecture de l'acquittement
    lire_smtp ();
}

==> libremail-2.3.tar.bz2: libremail/src/simulsmtp.c <==
/*
    Fichier simulsmtp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions permettant
    la simulation d'envoi de mails au serveur smtp
    Fonctionne mme sans connexion au rseau
*/

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "smtp.h"
#include "base64.h"


#define defaut_port_smtp  25  // port utilis par dfaut par les serveurs smtp

#define octet      unsigned char


/* Connexion au serveur smtp (mme fonction qu'en mode normal) */

int connect_smtp (char *infoserveur)
{
    octet *carinfo;       // caractre de infoserveur
    char  serv_smtp [80]; // nom du serveur smtp utilis
    int   port;           // numro du port smtp utilis
    FILE  *fich_auth;     // descripteur du fichier d'authentification smtp
    char  bufin [80];     // buffer pour lire le fichier d'authentification
    char  bufw [80];      // buffer d'envoi d'une ligne ou requte smtp
    int   caract;         // caractre du fichier d'authentification smtp
    int   retour;         // code de retour de la fonction


    // initialisation du numro de port par dfaut
    port = defaut_port_smtp;

    // si la ligne smtp de fichier de configuration fait
    // rfrence  un fichier d'authentification smtp
    if (*infoserveur == '>')
    {
        // se positionner sur le nom de ce fichier
        carinfo = infoserveur + 1;

        while (*carinfo == ' ' || *carinfo == '\t')
            carinfo ++;

        fich_auth = fopen (carinfo, "r");

        if (fich_auth)
        {
            // recuprer le nom du serveur smtp
            fgets (serv_smtp, 80, fich_auth);
            serv_smtp [strlen (serv_smtp) - 1] = '\0';

            // mmoriser les autres lignes particulires du fichier
            // d'authentification smtp (on s'arrte sur une ligne vide)
            while (fgets (bufin, 80, fich_auth) && *bufin != '\n')
            {
                bufin [strlen (bufin) - 1] = '\0';

                // numro de port spcifi ?
                if (startligne (bufin, "port"))
                {
                    // rcuprer le numro du port
                    carinfo = bufin + 4;

                    while (*carinfo == ' ' || *carinfo == '\t')
                        carinfo++;

                    port = atoi (carinfo);

                    // le vrifier sommairement
                    if (!port)
                    {
                        // "Numro de port smtp invalide"
                        affiche_err ("PORT_SMTP_INVALIDE");
                        retour = 0;
                    }
                }
            }

            // tentative de connexion au serveur smtp
            if (port)
                retour = connect_serv_smtp (serv_smtp, port);

            // si la connexion s'est bien droule
            if (retour)
            {
                // se positionner sur la premire ligne
                // du dialogue d'authentification
                while (fgets (bufin, 80, fich_auth) && *bufin == '\n')
                    ;

                // dialogue d'authentification
                do
                {
                    // mise en forme de la ligne lue
                    bufin [strlen (bufin) - 1] = '\0';

                    // si ligne  encoder base 64
                    if (startligne (bufin, "b64"))
                    {
                        // l'encoder avant de l'envoyer
                        encode64 (bufin + 4, bufw, strlen (bufin + 4));
                        puts (bufin);
                        envoie_smtp (bufw);
                    }
                    // sinon, envoyer directement la ligne
                    else
                        envoie_smtp (bufin);

                    // et lire la rponse du serveur
                    lire_smtp ();
                }
                // passage  la ligne suivante
                while (fgets (bufin, 80, fich_auth));
            }

            // librer le fichier d'authentification smtp
            fclose (fich_auth);
        }
        else
        {
            // message d'erreur
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", carinfo);
            retour = 0;
        }
    }
    // sinon simple connexion  un serveur smtp sans authentification
    else
    {
        // parcourir le nom du serveur smtp
        carinfo = infoserveur;

        while (*carinfo > ' ')
            carinfo ++;

        // si le nom du serveur smtp est suivi d'autres informations
        if (*carinfo)
        {
            // terminer le nom du serveur smtp
            *carinfo = '\0';

            // se positionner sur le numro de port ventuel
            do
                carinfo++;
            while (*carinfo == ' ' || *carinfo == '\t');
        }

        // si un numro de port semble prsent
        if (*carinfo)
            // le rcuprer
            port = atoi (carinfo);

        // si numro de port trouv ou si on a conserv le port par dfaut
        if (port)
            // tablir la connexion smtp avec le port choisi
            retour = connect_serv_smtp (infoserveur, port);

        // sinon message d'erreur
        else
        {
            // "Numro de port smtp invalide"
            affiche_err ("PORT_SMTP_INVALIDE");
            retour = 0;
        }

        // si la connexion smtp s'est bien droule
        if (retour)
        {
            // gnrer le message  helo nom_de_calculateur
            strcpy (bufw, "HELO ");
            gethostname (bufw + 5, sizeof (bufw) - 5);

            // identification de l'ordinateur connect
            envoie_smtp (bufw);
            lire_smtp ();
        }
    }

    // renvoyer le code de retour de la fonction
    return (retour);
}



/* Simulation de connexion russie au serveur smtp sans authentification */

int connect_serv_smtp (char *serveur, int port_smtp)
{
    char bufw [80];  // buffer d'envoi d'une ligne ou requte smtp


    printf ("Connexion au serveur %s sur le port %d\n", serveur, port_smtp);

    // simulation de la rponse du serveur smtp
    lire_smtp ();
    return (1);
}



/* teste si la ligne passe en paramtre commence par un mot cl particulier */

int startligne (char *ligne, char *motcle)
{
    int i;

    i = 0;

    // on teste caractre par caractre en ignorant la casse
    while (motcle [i] && tolower (ligne [i]) == tolower (motcle [i]))
        i++;

    // le mot cl doit tre suivi d'un blanc dans la ligne lue
    return (ligne [i] == ' ');
}



/* Simulation criture de donnes */

void envoie_smtp (char *buffer)
{
    puts (buffer);
}



/* Simulation lecture d'une ligne de donnes */

void lire_smtp ()
{
    strcpy (buf_lect, "250 OK");
}



/* Simulation dconnexion propre du serveur smtp */

void deconnect_smtp ()
{
    // envoi du message de deconnexion
    envoie_smtp ("QUIT");

    // lecture de l'acquittement
    lire_smtp ();
}
